@@ -44,15 +44,13 @@ class CustomHelpFormatter(argparse.RawDescriptionHelpFormatter):
         import re
         import textwrap as _textwrap
         textRows = text.splitlines()
-        for idx,line in enumerate(textRows):
-            search = re.search('\s*[0-9\-]{0,}\.?\s*', line)
-            if line.strip() is "":
+        for idx, line in enumerate(textRows):
+            search = re.search(r'\s*[0-9\-]{0,}\.?\s*', line)
+            if line.strip() == "":
                 textRows[idx] = " "
             elif search:
                 lWSpace = search.end()
-                lines = [self.__add_whitespace(i,lWSpace,x) for i,x in enumerate(_textwrap.wrap(line, width))]
+                lines = [self.__add_whitespace(i, lWSpace, x) for i, x in enumerate(_textwrap.wrap(line, width))]
                 textRows[idx] = lines
 
         return [item for sublist in textRows for item in sublist]
-
-
