@@ -109,6 +109,15 @@ def _subtract_mean(weight):
     return weight
 
 
+def _is_iterable(maybe_iterable):
+    try:
+        iter(maybe_iterable)
+    except TypeError:
+        return False
+    else:
+        return True
+
+
 class Correlation(object):
     """
     Base class for correlation functions.
@@ -177,6 +186,12 @@ class Correlation(object):
     indicates which variables should be read from the trajectory file.
     They will be available as self._pos, self._pos_unf, self._vel.
     """
+    _symmetric = True
+    """
+    If nbodies>1, a symmetric correlation function is invariant under
+    the exchange between _pos_0 and _pos_1. In that case, _symmetric
+    is True.
+    """
 
     def __init__(self, trj, grid, output_path=None, norigins=None, fix_cm=False):
         # Accept a trajectory-like instance or a path to a trajectory
@@ -192,7 +207,10 @@ class Correlation(object):
         self.comments = None  # can be modified by user at run time
         self.tag = ''
         self.tag_description = 'the whole system'
-        self.output_path = output_path if output_path is not None else core.pp_output_path
+        if self.trajectory.filename is None:
+            self.output_path = None
+        else:
+            self.output_path = output_path if output_path is not None else core.pp_output_path
         self.skip = adjust_skip(self.trajectory, norigins)
 
         # Callbacks
@@ -352,9 +370,6 @@ class Correlation(object):
             if hasattr(th[0].particle[0], 'position_unfolded'):
                 # Unfolded positions are present in the trajectory
                 for s in progress(th):
-                    # Apply filter if there is one
-                    if len(self._cbk) > 0:
-                        s = self._cbk[0](s, *self._cbk_args[0], **self._cbk_kwargs[0])
                     # Fixing the CM must be done explicitly using
                     # particle.position_unfolded because atooms.system
                     # methods work with particle.position
@@ -370,6 +385,10 @@ class Correlation(object):
                         # Subtract it
                         for p in s.particle:
                             p.position_unfolded -= cm
+                    # Apply filter if there is one
+                    # This must be done after the CM has been fixed
+                    if len(self._cbk) > 0:
+                        s = self._cbk[0](s, *self._cbk_args[0], **self._cbk_kwargs[0])
                     self._pos_unf.append(s.dump('particle.position_unfolded'))
             else:
                 for s in progress(th):
@@ -576,16 +595,8 @@ class Correlation(object):
         The default is defined by core.pp_output_path, which currently
         looks like '{trajectory.filename}.pp.{symbol}.{tag}'
         """
-        def is_iterable(maybe_iterable):
-            try:
-                iter(maybe_iterable)
-            except TypeError:
-                return False
-            else:
-                return True
-
         # Pack grid and value into arrays to dump
-        if is_iterable(self.grid[0]) and len(self.grid) == 2:
+        if _is_iterable(self.grid[0]) and len(self.grid) == 2:
             x = numpy.array(self.grid[0]).repeat(len(self.value[0]))
             y = numpy.array(self.grid[1] * len(self.grid[0]))
             z = numpy.array(self.value).flatten()
@@ -621,13 +632,14 @@ class Correlation(object):
         # and make sure the path to the output file exists
         import os
         from atooms.core.utils import mkdir
-        mkdir(os.path.dirname(self._output_file))
-        with open(self._output_file, 'w') as fh:
-            fh.write(comments)
-            if len(analysis) > 0:
-                fh.write(analysis)
-            numpy.savetxt(fh, dump, fmt="%g")
-            fh.flush()
+        if self._output_file is not None:
+            mkdir(os.path.dirname(self._output_file))
+            with open(self._output_file, 'w') as fh:
+                fh.write(comments)
+                if len(analysis) > 0:
+                    fh.write(analysis)
+                numpy.savetxt(fh, dump, fmt="%g")
+                fh.flush()
 
     def do(self, update=False):
         """
@@ -650,3 +662,16 @@ class Correlation(object):
 
     def __call__(self):
         self.do()
+
+    def show(self, now=True):
+        try:
+            import matplotlib.pyplot as plt
+        except ImportError:
+            return
+
+        if not _is_iterable(self.grid[0]):
+            plt.plot(self.grid, self.value)
+            plt.ylabel(self.short_name)
+            plt.xlabel(self.grid_name[0])
+        if now:
+            plt.show()
