@@ -109,7 +109,7 @@ class RadialDistributionFunctionLegacy(Correlation):
                 # Redefine grid to extend up to L
                 # The grid will be cropped later to L/2
                 # This retains the original dr
-                self.grid = linear_grid(self.dr/2, min(system.cell.side), self.dr)
+                self.grid = linear_grid(self.dr/2, max(system.cell.side)*ndims**0.5, self.dr)
             else:
                 # If there is no cell, then rmax must have been given
                 # This retains the original dr
@@ -119,6 +119,7 @@ class RadialDistributionFunctionLegacy(Correlation):
                     self.grid = linear_grid(self.dr/2, self.dr*1000, self.dr)
 
         # Internal max distance (/= user provided rmax)
+        # TODO: not used
         self._rmax = self.grid[-1]
 
     def _compute(self):
@@ -193,14 +194,22 @@ class RadialDistributionFunctionFast(RadialDistributionFunctionLegacy):
     with a spacing of `dr`. Here, L is the side of the simulation cell
     along the x axis at the first step.
 
+    If `cell` is not `None`, then
+    - if all directions are periodic, we do the standard calculation
+    - if all directions are non periodic, we crop the bulk of the cell
+      (this can be used to analyze a subsystem)
+    - (TODO) if only some directions are non periodic, we shouldadd
+      means to bin g(r) according to some callback of system
+
     Additional parameters:
     ----------------------
 
-    - norigins: controls the number of trajectory frames to compute
+    - `norigins`: controls the number of trajectory frames to compute
       the time average
     """
 
     def _compute(self):
+        import sys
         from atooms.postprocessing.realspace_wrap import compute
         from atooms.postprocessing.linkedcells import LinkedCells
 
@@ -211,8 +220,6 @@ class RadialDistributionFunctionFast(RadialDistributionFunctionLegacy):
         gr_all = []
 
         # Grid setup.
-        # Get side of cell. If the system does not have a cell,
-        # wrap it in an infinite cell and estimate a reasonable grid.
         system = self.trajectory.read(0)
         ndims = system.number_of_dimensions
         self._setup_grid(system)
@@ -225,6 +232,7 @@ class RadialDistributionFunctionFast(RadialDistributionFunctionLegacy):
         # - memory footprint is < ~1Gb
         # These tests are done of the first framce
         # TODO: if memory footprint is surpassed skip particles
+        # TODO: fix fluctating cell case!
         linkedcells = None
         if self.rmax > 0.0 and system.cell is not None:
             n_0 = len(self._pos_0[0])
@@ -244,52 +252,62 @@ class RadialDistributionFunctionFast(RadialDistributionFunctionLegacy):
         # Main loop for average
         origins = range(0, len(self.trajectory), self.skip)
         for i in progress(origins):
+            # Shortcuts
+            pos_0 = self._pos_0[i]
+            pos_1 = self._pos_1[i]
+
             # Skip if there are no particles
-            if len(self._pos_0[i]) == 0 or len(self._pos_1[i]) == 0:
+            if len(pos_0) == 0 or len(pos_1) == 0:
                 continue
 
             # Switch between self and distinct calculation
-            distinct = self._pos_0 is not self._pos_1
+            distinct = pos_0 is not pos_1
+            remove_self_term = False
 
             # Store side
             system = self.trajectory.read(i)
-            if system.cell is not None:
-                side = system.cell.side
-            else:
-                import sys
-                side = numpy.ndarray(ndims, dtype=float)
-                side[:] = sys.float_info.max
+            if system.cell is not None and hasattr(system.cell, 'periodic') and \
+               sum(system.cell.periodic) == 0 and self.rmax > 0:
+                # With a non-periodic cell, which just bounds the physical
+                # domain, we crop particles away from the surface (within rmax)
+                assert self.rmax > 0, 'provide rmax>0'
+                # Booleans do not work here, so we just use integers,
+                # which are 1 is particles are on the surface and 0 otherwise
+                mask = compute.on_surface_c(pos_0, system.cell.side, self.rmax)
+                pos_0 = pos_0[mask == 0, :]
+                # Force distinct calculation. This gives a spurious signal at r=0, which we remove
+                distinct = True
+                remove_self_term = True
 
             # When using linked cells, we precalculate the neighbors
+            # This will only happen if the system is within a cell (and all directions are periodic)
             if linkedcells:
-                if self._pos_0 is self._pos_1:
-                    neighbors, number_of_neighbors = linkedcells.compute(side, self._pos_0[i], as_array=True)
+                if not distinct:
+                    neighbors, number_of_neighbors = linkedcells.compute(system.cell.side, pos_0, as_array=True, periodic=system.cell.periodic)
                 else:
-                    neighbors, number_of_neighbors = linkedcells.compute(side, self._pos_0[i], self._pos_1[i], as_array=True)
+                    neighbors, number_of_neighbors = linkedcells.compute(system.cell.side, pos_0, pos_1, as_array=True, periodic=system.cell.periodic)
 
-            # Shortcuts
-            if distinct:
-                pos_0 = self._pos_0[i]
-                pos_1 = self._pos_1[i]
+            # If there is no cell and anyway along non-periodic directions,
+            # we replace sides with infty to work with f90 kernels (which apply PBC)
+            if system.cell is not None:
+                side = system.cell.side.copy()
+                # TODO: fix partly periodic boundaries
+                assert sum(system.cell.periodic) in [0, ndims], \
+                    'partly periodic cells are not supported yet'
+                for i in range(len(side)):
+                    if not system.cell.periodic[i]:
+                        side[i] = sys.float_info.max
             else:
-                pos_0 = self._pos_0[i]
-                pos_1 = pos_0
-
-            # With a non-periodic cell, which just bounds the physical
-            # domain, we must crop particles close to the surface
-            # self_term = 0
-            if hasattr(system.cell, 'periodic') and not numpy.any(system.cell.periodic):
-                # mask = numpy.ndarray(pos_0.shape[1], dtype=numpy.bool)
-                # TODO: bool does not work
-                mask = compute.on_surface_c(pos_0, side, self.rmax)
-                mask = mask == 1
-                pos_0 = pos_0[:, mask]
-                # Force distinct calculation
-                distinct = True
+                # If the system does not have a cell, wrap it in an
+                # infinite cell. This way we can still use PBC in f90 kernels
+                side = sys.float_info.max * numpy.ones(ndims, dtype=float)
 
             # Store number of particles for normalization
             N_0.append(pos_0.shape[0])
-            N_1.append(pos_1.shape[0])
+            if distinct:
+                N_1.append(pos_1.shape[0])
+            else:
+                N_1.append(pos_0.shape[0])
 
             # Compute g(r)
             if not distinct:
@@ -306,6 +324,11 @@ class RadialDistributionFunctionFast(RadialDistributionFunctionLegacy):
             # Damned copies in python
             gr_all.append(gr.copy())
 
+        # import matplotlib.pyplot as plt
+        # plt.plot(pos_0[:, 0], pos_0[:, 1], 'o')
+        # plt.plot(pos_1[:, 0], pos_1[:, 1], '+')
+        # plt.show()
+        
         # Normalization
         # Array r is used to compute shells, we thus use the bin boundaries
         r = bins - (bins[1] - bins[0]) / 2
@@ -314,7 +337,6 @@ class RadialDistributionFunctionFast(RadialDistributionFunctionLegacy):
         if system.cell is not None:
             volume = system.cell.volume
         else:
-            # TODO: this only works with recent atooms
             volume = len(system.particle) / system.density
         if ndims == 2:
             vol = math.pi * (r[1:]**2 - r[:-1]**2)
@@ -327,14 +349,17 @@ class RadialDistributionFunctionFast(RadialDistributionFunctionLegacy):
 
         rho = N_1 / volume
         norm = rho * vol * N_0
+
         if not distinct:
             # We used Newton III
             norm /= 2
         gr = numpy.average(gr_all, axis=0)
 
-        # We crop the last entry
-        self.grid = bins[:-1]
+        # Normalize and clean up boundaries
         self.value = gr[:-1] / norm
+        self.grid = bins[:-1]
+        if remove_self_term:
+            self.value[0] = self.value[1]
 
         # Restrict distances to L/2 (in last frame) or rmax
         if self.rmax > 0:
